--[[
    Helper functions
    $Revision: 77587 $
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local MAJOR_VERSION = "LibLordFarlander-UI-2.0";
local MINOR_VERSION = "$Rev: 77587 $";

if( not LibStub( "LibKeyBound-1.0", true ) ) then
    error( MAJOR_VERSION .. " requires LibKeyBound-1.0" );
    return;
end--if

if( not LibStub( "LibStickyFrames-2.0", true ) ) then
    error( MAJOR_VERSION .. " requires LibStickyFrames-2.0" );
    return;
end--if

local LibLordFarlander_UI, oldminor = LibStub:NewLibrary( MAJOR_VERSION, MINOR_VERSION );
if( not LibLordFarlander_UI ) then
    return;
end--if

function LibLordFarlander_UI.SetupForUI( class )
    local LibKeyBound = LibStub( "LibKeyBound-1.0" );

    -- LibKeyBound support
    LibKeyBound.RegisterCallback( class, "LIBKEYBOUND_ENABLED", LibLordFarlander_UI.LIBKEYBOUND_ENABLED, class );
    LibKeyBound.RegisterCallback( class, "LIBKEYBOUND_DISABLED", LibLordFarlander_UI.LIBKEYBOUND_DISABLED, class );
    LibKeyBound.RegisterCallback( class, "LIBKEYBOUND_MODE_COLOR_CHANGED", LibLordFarlander_UI.LIBKEYBOUND_MODE_COLOR_CHANGED, class );
    class.keyBoundMode = false;
end--LibLordFarlander_UI.SetupUI( class )

function LibLordFarlander_UI.GetButtonSettingsDefaults()
    return {
            AutoHideButton = true,
            LockButton = false,
            ButtonGroup = {
                SkinID = "Blizzard",
                Gloss = nil,
                Backdrop = nil,
                Colors = {},
            },
            FadeOptions = {
                fadeOut = false,
                fadeOutUpdateTime = 0.1,
                fadeOutCancelInCombat = true,
                fadeOutCancelOnShift = false,
                fadeOutCancelOnCtrl = false,
                fadeOutCancelOnAlt = false,
                alpha = 1.0,
                fadeOutDelay = 0.5,
                fadeOutAlpha = 0.2,
                fadeOutTime = 0.5,
            },
            posX = 20,
            posY = 20,
        };
end--LibLordFarlander_UI.GetButtonSettingsDefaults()

function LibLordFarlander_UI.CreateButton( class, groupName, defaultImage, ButtonFacadeGroup )
    local button = CreateFrame( "Button", groupName .. "_Button", UIParent, "SecureActionButtonTemplate, ActionButtonTemplate" );
    local ButtonSettings = class.db.profile.Button;
    local LibStickyFrames = LibStub( "LibStickyFrames-2.0" );
    local LibButtonFacade = LibStub( "LibButtonFacade", true );

    if( button == nil ) then
        return nil;
    end--if
    button:EnableMouse( true );
    button:SetMovable( true );
    button:RegisterForDrag( "LeftButton" );
    button:RegisterForClicks( "AnyUp" );
    
    local x, y, s = ButtonSettings.posX, ButtonSettings.posY, button:GetEffectiveScale();

    button:ClearAllPoints()
    button:SetPoint( "BOTTOMLEFT", UIParent, "BOTTOMLEFT", x / s, y / s );

    button.class = class;
    button.tooltip = GameTooltip;
    button.cooldown = _G[button:GetName() .. "Cooldown"];
    button.GetHotkey = function( self ) return GetBindingKey( "BINDING_NAME_CLICK " .. self:GetName() .. ":LeftButton" ) end;
    button.ActionName = groupName;
    button.GetActionName = function( self ) return self.ActionName end;
    button.HideEx = LibLordFarlander_UI.HideEx;
    button.ShowEx = LibLordFarlander_UI.ShowEx;
    button.SetImage = LibLordFarlander_UI.SetButtonImage;
    button.OutOfCombatReset = LibLordFarlander_UI.OutOfCombatButtonReset;
    button.icon = _G[button:GetName() .. "Icon"];
    button:SetScript( "OnEnter",
        function( button )
            if( button.class and button.class.SetGameTooltips ) then
                button.class:SetGameTooltips();
            end--if
        end );
    button:SetScript( "OnLeave",
        function( button )
            if( button.tooltip ) then
                button.tooltip:Hide();
            end--if
        end );
    button:SetScript( "OnUpdate",
        function( button, _, elapsed )
            if( button.class and button.class.OnUpdate ) then
                button.class:OnUpdate( elapsed );
            end--if
        end );
    button:SetScript( "OnDragStart", LibLordFarlander_UI.OnDragStart );
    button:SetScript( "OnDragStop", LibLordFarlander_UI.OnDragStop );

--    self.button.FadeOnUpdate = LibLordFarlander.ButtonFadeOnUpdate;
    LibStickyFrames:SetFrameGroup( button, true );
    LibStickyFrames:SetFrameEnabled( button, true );
    LibStickyFrames:SetFrameText( button, groupName );
    LibStickyFrames.RegisterCallback( class, "OnStickToFrame", LibLordFarlander_UI.OnStickToFrame, class );
    LibStickyFrames.RegisterCallback( class, "OnStopFrameMoving", LibLordFarlander_UI.OnStickToFrame, class );
    if( ButtonSettings.StickToFrame and _G[ButtonSettings.StickToFrame] ) then
        LibStickyFrames:SetFramePoints( button, ButtonSettings.StickPoint, _G[ButtonSettings.StickToFrame], ButtonSettings.StickToPoint, ButtonSettings.StickToX, ButtonSettings.StickToY );
    end--if
    if( LibButtonFacade ) then
        local ButtonFacadeGroup = ButtonSettings.ButtonGroup;

        LibButtonFacade:Group( groupName ):AddButton( button );
        LibButtonFacade:Group( groupName ):Skin( ButtonFacadeGroup.SkinID, ButtonFacadeGroup.Gloss, ButtonFacadeGroup.Backdrop, ButtonFacadeGroup.Colors );
        LibButtonFacade:RegisterSkinCallback( groupName, LibLordFarlander_UI.SkinChanged, class );
    end--if
    button:SetImage( defaultImage );
    return button;
end

function LibLordFarlander_UI.OutOfCombatButtonReset( button )
    if( button.NeedToShow ) then
        button.NeedToShow = false;
        button:ShowEx();
    end--if
    if( button.NeedToHide ) then
        button.NeedToHide = false;
        button:HideEx();
    end--if
end--LibLordFarlander_UI.OnDragStop( button )

function LibLordFarlander_UI.OnProfileChanged( self )
    local LibStickyFrames = LibStub( "LibStickyFrames-2.0" );
    local LibButtonFacade = LibStub( "LibButtonFacade", true );
    local button = self.button;
    local ButtonSettings = self.db.profile.Button;
    local x, y, s = ButtonSettings.posX, ButtonSettings.posY, button:GetEffectiveScale();

    button:ClearAllPoints()
    button:SetPoint( "BOTTOMLEFT", UIParent, "BOTTOMLEFT", x / s, y / s );
    if( ButtonSettings.StickToFrame and _G[ButtonSettings.StickToFrame] ) then
        LibStickyFrames:SetFramePoints( button, ButtonSettings.StickPoint, _G[ButtonSettings.StickToFrame], ButtonSettings.StickToPoint, ButtonSettings.StickToX, ButtonSettings.StickToY );
    end--if
    if( LibButtonFacade ) then
        local ButtonFacadeGroup = ButtonSettings.ButtonGroup;

        LibButtonFacade:Group( groupName ):AddButton( button );
        LibButtonFacade:Group( groupName ):Skin( ButtonFacadeGroup.SkinID, ButtonFacadeGroup.Gloss, ButtonFacadeGroup.Backdrop, ButtonFacadeGroup.Colors );
        LibButtonFacade:RegisterSkinCallback( groupName, LibLordFarlander_UI.SkinChanged, self );
    end--if
end--LibLordFarlander_UI.OnProfileChanged( self )

function LibLordFarlander_UI.OnDragStart( button )
    LibStub( "LibStickyFrames-2.0" ):StartFrameMoving( button );
end--LibLordFarlander_UI.OnDragStart( button )

function LibLordFarlander_UI.OnDragStop( button )
    LibStub( "LibStickyFrames-2.0" ):StopFrameMoving( button );
end--LibLordFarlander_UI.OnDragStop( button )

--For LibStickyFrames support
function LibLordFarlander_UI.OnStickToFrame( event, frame, point, stickToFrame, stickToPoint, stickToX, stickToY )
    if( frame.class ) then
        local x, y = frame:GetLeft(), frame:GetBottom()
        local s = frame:GetEffectiveScale()
        local Button = frame.class.db.profile.Button;

        LibStub( "LibStickyFrames-2.0" ):SetFramePoints( frame, point, stickToFrame, stickToPoint, stickToX, stickToY );
        Button.posX = x * s;
        Button.posY = y * s;
        Button.StickPoint = point;
        Button.StickToFrame = stickToFrame and stickToFrame:GetName() or nil;
        Button.StickToPoint = stickToPoint;
        Button.StickToX = stickToX;
        Button.StickToY = stickToY;
    end--if
end--LibLordFarlander_UI.OnStickToFrame( event, frame, point, stickToFrame, stickToPoint, stickToX, stickToY )

--For ButtonFacade support
function LibLordFarlander_UI.SkinChanged( self, SkinID, Gloss, Backdrop, barKey, buttonKey, Colors )
    local ButtonFacadeGroup = self.db.profile.Button.ButtonGroup;

    ButtonFacadeGroup.SkinID = SkinID;
    ButtonFacadeGroup.Gloss = Gloss;
    ButtonFacadeGroup.Backdrop = Backdrop;
    ButtonFacadeGroup.Colors = Colors;
end--LibLordFarlander_UI.SkinChanged( SkinID, Gloss, Backdrop, barKey, buttonKey, Colors )

function LibLordFarlander_UI.HideEx( self )
    if( InCombatLockdown() ) then
        self.NeedToShow = false;
        self.NeedToHide = true;
    else
        self:Hide();
        LibStub( "LibStickyFrames-2.0" ):SetFrameHidden( self, true );
    end--if
end--LibLordFarlander_UI.HideEx( self )

function LibLordFarlander_UI.ShowEx( self )
    if( InCombatLockdown() ) then
        self.NeedToShow = true;
        self.NeedToHide = false;
    else
        self:Show();
        LibStub( "LibStickyFrames-2.0" ):SetFrameHidden( self, false );
    end--if
end--LibLordFarlander_UI.ShowEx( self )

--For LibKeyBound support
function LibLordFarlander_UI.ColorButton( self )
    if( self.keyBoundMode ) then
        self.button:SetBackdropColor( LibStub( "LibKeyBound-1.0" ):GetColorKeyBoundMode() );
    end--if
end--LibLordFarlander_UI.ColorButton( self )

--For LibKeyBound support
function LibLordFarlander_UI.LIBKEYBOUND_ENABLED( self )
    self.keyBoundMode = true;
    LibLordFarlander_UI.ColorButton( self );
end--LibLordFarlander_UI.LIBKEYBOUND_ENABLED( self )

--For LibKeyBound support
function LibLordFarlander_UI.LIBKEYBOUND_DISABLED( self )
    self.keyBoundMode = false;
    LibLordFarlander_UI.ColorButton( self );
end--LibLordFarlander_UI.LIBKEYBOUND_DISABLED( self )

--For LibKeyBound support
function LibLordFarlander_UI.LIBKEYBOUND_MODE_COLOR_CHANGED( self )
    LibLordFarlander_UI.ColorButton( self )
end--LibLordFarlander_UI.LIBKEYBOUND_MODE_COLOR_CHANGED( self )

--For LibKeyBound support
function LibLordFarlander_UI.AfterSetTooltip( button )
    if( button.GetHotkey ) then
       LibStub( "LibKeyBound-1.0" ):Set( button );
    end--if
end--LibLordFarlander_UI.AfterSetTooltip( button )

-- Notes:
-- Sets the image for a button.
-- To set the image of the button's icon, make sure the button has an icon member set to its icon.
-- Arguments:
--     button - The button
--     string - Path to the image to set the button to
function LibLordFarlander_UI.SetButtonImage( button, img )
    if( button.icon ) then
        button.icon:SetTexture( img );
    else
        button:SetNormalTexture( img );
        button:SetHighlightTexture( img );
    end--if
end--LibLordFarlander_UI.SetButtonImage( button, img )

function LibLordFarlander_UI.UpdateFadeOut( button, fadingOptions )
    if( fadingOptions.fadeOut ) then
        local cancelFade = ( InCombatLockdown() and fadingOptions.fadeOutCancelInCombat ) or
            MouseIsOver( button ) or
            ( IsShiftKeyDown() and fadingOptions.fadeOutCancelOnShift ) or
            ( IsControlKeyDown() and fadingOptions.fadeOutCancelOnCtrl ) or
            ( IsAltKeyDown() and fadingOptions.fadeOutCancelOnAlt );

        if( cancelFade ) then
            button:SetAlpha( fadingOptions.alpha );
            button.faded = nil
            button.fadeOutDelay = fadingOptions.fadeOutDelay
        elseif( not button.faded ) then
            local startAlpha = fadingOptions.alpha;
            local fadeOutAlpha = fadingOptions.fadeOutAlpha or 0;
            local fadeOutChunks = ( fadingOptions.fadeOutTime or 10 ) / fadingOptions.fadeOutUpdateTime;
            local decrement = ( startAlpha - fadeOutAlpha ) / fadeOutChunks;
            local alpha = button:GetAlpha() - decrement;

            if( alpha < fadeOutAlpha ) then
                alpha = fadeOutAlpha;
            end
            if( alpha > fadeOutAlpha ) then
                button:SetAlpha( alpha );
            else
                button:SetAlpha( fadeOutAlpha );
                button.faded = true;
            end--if
        end--if
    end--if
end--LibLordFarlander_UI.UpdateFadeOut( button, fadingOptions )

function LibLordFarlander_UI.ButtonFadeOnUpdate( button, elapsed, fadingOptions )
    if( fadingOptions.fadeOut ) then
        button.elapsed = button.elapsed + elapsed;
        if( button.fadeOutDelay ) then
           if( button.elapsed < button.fadeOutDelay ) then
              return;
           else
               button.elapsed = button.elapsed - self.fadeOutDelay;
               button.fadeOutDelay = nil;
           end--if
        end--if
        if( button.elapsed > fadingOptions.fadeOutUpdateTime ) then
           LibLordFarlander_UI.UpdateFadeOut( button, fadingOptions )
           button.elapsed = 0
        end--if
    end--if
end--LibLordFarlander_UI.ButtonFadeOnUpdate( button, elapsed, fadingOptions )
